% Numerical example 2 in the paper:
%
%   [1] Ding Lu, Yangfeng Su, and Zhaojun Bai, Stability Analysis 
%       of the two-level orthogonal Arnoldi procedure,
%       SIAM J. Matrix Anal. Appl., 2016. 37(1): 195--214.
%
% Ding Lu, Fudan Univ., 2012/2/8.
%
clc; clear all; close all;

load gyro_beta10to7.mat;

N = size(M,1); % problem size
s = 2*pi*sqrt(-1) *fq;
s0 = 1.05*10^5; % expansion point: as in Y.T.L's test 

% Shifting coefficient matrices
if s0==0
    TM = M; TE = E; TK = K;
else
    TM = M; TE = 2*s0*M + E; TK = s0^2*M + s0*E + K;
end

[L, U, P, Q] = lu(TK);  % LU decomp
v0 = Q*(U\(L\(P*b)));

av = @(p) -Q*(U\(L\(P*(TE*p))));
bv = @(q) -Q*(U\(L\(P*(TM*q))));


% Base matrix by TOAR
[Vn, RP, RQ, Hk, rk] = TOAR(av, bv, N, 200, v0);

% Reduced transfor functions by TOAR
hht=[];
for j = [20, 30,90,200]
    Mn = Vn(:,1:j)'*M*Vn(:,1:j);
    En = Vn(:,1:j)'*E*Vn(:,1:j);
    Kn = Vn(:,1:j)'*K*Vn(:,1:j);
    ln = Vn(:,1:j)'*l;
    bn = Vn(:,1:j)'*b;
    hh= zeros( size(s) );
    for i = 1:length(s)
      hh(i) = ln'*( (s(i)^2*Mn+s(i)*En+Kn)\bn );
    end
    hht = [hht;hh];
end


% Base matrix by SOAR
[VSn, PP, TT] = SOAR(av, bv, N, 200, v0);

% Reduced transfor function by SOAR
hhs=[];
for j = [20, 30,90,200]
    Mn = VSn(:,1:j)'*M*VSn(:,1:j);
    En = VSn(:,1:j)'*E*VSn(:,1:j);
    Kn = VSn(:,1:j)'*K*VSn(:,1:j);
    ln = VSn(:,1:j)'*l;
    bn = VSn(:,1:j)'*b;
    hh= zeros( size(s) );
    for i = 1:length(s)
      hh(i) = ln'*( (s(i)^2*Mn+s(i)*En+Kn)\bn );
    end
    hhs = [hhs;hh];
end

figure('Position', [0, 0, 1500, 500]); 

% transfer functon plot 
g0=subplot(1,3,1);
set(g0, 'position', [0.06 0.15 .3 .7],'FontSize',16);
loglog(fq,(abs(h)),'Color',[0,.5,0],'LineStyle','-','LineWidth',2);
hold on;
loglog(fq,(abs(hhs(4,:))),'Color',[1,0,0],'LineStyle','-.','LineWidth',2);
loglog(fq,(abs(hht(4,:))),'Color',[0,0,1],'LineStyle',':','LineWidth',2);
legend( 'Exact','SOAR','TOAR','Location','NorthEast');
xlabel('Frequency (Hz)');
ylabel('Magnitudes');
 
% sample a subset of frequency to produce less noisy error plot
inc = 6; % sampling interval
fq = fq([1:inc:end]);
hhs = hhs(:,[1:inc:end]);
hht = hht(:,[1:inc:end]);
h = h([1:inc:end]);

% SOAR error
g1=subplot(1,3,2);
set(g1, 'position', [0.43 0.15 .25 .7],'FontSize',16);
loglog(fq,(abs(h-hhs(1,:))./abs(h)),'Color',[0,0,1],'LineStyle',':','LineWidth',2)
hold on;
loglog(fq,(abs(h-hhs(2,:))./abs(h)),'Color',[0,.5,0],'LineStyle','-.','LineWidth',2)
loglog(fq,(abs(h-hhs(3,:))./abs(h)),'Color',[1,.5,0],'LineStyle','--','LineWidth',2)
loglog(fq,(abs(h-hhs(4,:))./abs(h)),'Color',[1,0,0],'LineStyle','-','LineWidth',2)
legend('k=20','k=30','k=90','k=200','Location','NorthWest');
xlabel('Frequency (Hz)');
ylabel('Relative Error');
ylim([10^-13,10^3])
title('SOAR');
set(gca,'YGrid','on','YMinorGrid','on');

% TOAR error
g2=subplot(1,3,3);
set(g2, 'position', [0.72 0.15 .25 .7],'FontSize',16);
loglog(fq,(abs(h-hht(1,:))./abs(h)),'Color',[0,0,1],'LineStyle',':','LineWidth',2)
hold on;
loglog(fq,(abs(h-hht(2,:))./abs(h)),'Color',[0,.5,0],'LineStyle','-.','LineWidth',2)
loglog(fq,(abs(h-hht(3,:))./abs(h)),'Color',[1,.5,0],'LineStyle','--','LineWidth',2)
loglog(fq,(abs(h-hht(4,:))./abs(h)),'Color',[1,0,0],'LineStyle','-','LineWidth',2)
legend('k=20','k=30','k=90','k=200','Location','NorthWest');
xlabel('Frequency (Hz)');
ylim([10^-13,10^3])
title('TOAR');
set(gca,'YGrid','on','YMinorGrid','on');

return
% END
